package example.cmp.single_table;

/**
 * Implementation of the HouseBean.  Each instance of HouseBean
 * maps to a table entry of "house", where house is defined as
 *
 * <pre>
 * CREATE TABLE house (
 *   name VARCHAR(250),
 *   points INTEGER
 * )
 * </pre>
 *
 * <p>The bean implementation is abstract and the get and set methods
 * for the managed fields are abstract to take advantage of Resin-CMP's
 * container-managed persistence.  All business methods must be
 * fully implemented.  Resin-CMP will create the implementation of
 * the abstract methods.
 *
 * <p>Resin-CMP will cache the bean instance.  Once the instance is loaded,
 * it can avoid calling the database for any further calls.
 *
 * <p>HouseBean extends the AbstractEntityBean implementation.
 * AbstractEntityBean is a stub EntityBean implementation with default
 * methods to make life a little more sane for simple beans.
 */
abstract public class HouseBean extends com.caucho.ejb.AbstractEntityBean {
  /**
   * Returns the house name.  The name is the primary key.
   */
  abstract public String getName();
  /**
   * Sets the house name.
   *
   * <p>In this example, the setName method is optional
   * since it doesn't have a create method.  setName would be used in a
   * create method to set the primary key for the new item.  Beans must
   * never call setName in any other situation than the create method.
   *
   * <p>The set methods for the primary key are never exposed in the
   * local interface.
   *
   * @param name the name for a new house instance.
   */
  abstract public void setName(String name);
  /**
   * Returns the number of points for the house.  Resin-CMP will fill
   * in the implementation.
   *
   * <p>With Resin-CMP's caching, this will normally not require a new
   * database call after the first request.
   */
  abstract public int getPoints();
  /**
   * Sets the number of points for the house.  This method is hidden from
   * clients since it's not exposed in the House interface.  It is only
   * called by the business method <code>addPoints</code>.  Clients must
   * call <code>addPoints</code> to change the house's points.
   *
   * @param points the new number of points for the house.
   */
  abstract public void setPoints(int points);
  /**
   * Business method to add a number of points to the house.  Changes
   * to database fields will often be encapsulated in a business
   * method.
   *
   * <p>Because the getPoints and setPoints calls are contained in the
   * same business method call, it's in the same transaction.  That
   * means the update will be consistent even with multiple threads.
   *
   * <p>The automatic transaction is like a synchronize call, but is
   * more efficient and potentially capable of handling distributed
   * updates.
   *
   * @param delta the points to add or subtract.
   */
  public void addPoints(int delta)
  {
    setPoints(getPoints() + delta);
  }
}
