package example.cmp.single_table;

import java.io.*;

import javax.ejb.*;
import javax.naming.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * Example database client servlet using the house table.  Typically,
 * Resin-CMP clients will be servlets, their associated classes and JSPs.
 *
 * <h3>JNDI client configuration</h3>
 *
 * <code><pre>
 * &lt;jndi-link>
 *   &lt;jndi-name>java:comp/env/cmp&lt;/jndi-name>
 *   &lt;jndi-factory>com.caucho.ejb.LocalContextFactory&lt;/jndi-factory>
 *   &lt;init-param java.naming.provider.url="http://localhost:8080/cmp/example"/>
 * &lt;/jndi-link>
 * </pre></code>
 */
public class HouseServlet extends HttpServlet {
  /**
   * The servlet stores the home interface after the initial lookup.
   * Since the home interface never changes, caching the lookup will save
   * some performance.
   */
  private HouseHome houseHome;

  /**
   * The init method looks up the HouseHome interface using JNDI and
   * stores it in a servlet variable.
   */
  public void init()
    throws ServletException
  {
    try {
      /*
       * Look up the local CMP context using JNDI.
       * The context contains the CMP beans in the local web-app
       */
      Context cmp = (Context) new InitialContext().lookup("java:comp/env/cmp");
      /*
       * Look up the single_table home.  "single_table" is the ejb-name
       * in the deployment descriptor.
       */
      houseHome = (HouseHome) cmp.lookup("single_table");
    } catch (Exception e) {
      throw new ServletException(e);
    }
  }

  /**
   * Returns the points for the named houses.  The House interface is
   * found using the findByPrimaryKey method.
   *
   * <p>A more complicated example would define a findAll method to
   * return all the current houses rather than hard-coding them in
   * the servlet.
   *
   * @param request the servlet request object.
   * @param response the servlet response object.
   */
  public void doGet(HttpServletRequest request,
                    HttpServletResponse response)
    throws IOException, ServletException
  {
    try {
      PrintWriter out = response.getWriter();

      /*
       * The known houses are predefined to simplify the example by
       * avoiding defining a findAll method.
       */
      String []houses = new String[] { "Gryffindor", "Slytherin",
                                       "Ravenclaw", "Hufflepuff" };

      response.setContentType("text/html");

      out.println("<table>");

      for (int i = 0; i < houses.length; i++) {
        /*
         * Look up the remote object from the home interface.
         */
        House house = houseHome.findByPrimaryKey(houses[i]);
  
        out.println("<tr><td>");
        /*
         * Get the house's name.  Although the call is relatively
         * lightweight, it's still slower than a normal method call.
         */
        out.print(house.getName());
        
        out.print("<td>");
        /*
         * And the points.  Each call will be in its own transaction, so
         * it would be conceivable that the points value could change
         * between the calls.
         */
        out.print(house.getPoints());

        // Each call will be in its
      }

      out.println("</table>");
    } catch (FinderException e) {
      throw new ServletException(e);
    }
  }
}
